use leptos::{prelude::*, task::spawn_local};
use leptos_meta::{provide_meta_context, MetaTags, Stylesheet, Title};
use leptos_router::{
    components::{Route, Router, Routes},
    StaticSegment,
};

pub fn shell(options: LeptosOptions) -> impl IntoView {
    view! {
        <!DOCTYPE html>
        <html lang="en">
            <head>
                <meta charset="utf-8"/>
                <meta name="viewport" content="width=device-width, initial-scale=1"/>
                <AutoReload options=options.clone() />
                <HydrationScripts options/>
                <MetaTags/>
            </head>
            <body>
                <App/>
            </body>
        </html>
    }
}

#[component]
pub fn App() -> impl IntoView {
    // Provides context that manages stylesheets, titles, meta tags, etc.
    provide_meta_context();

    view! {
        // injects a stylesheet into the document <head>
        // id=leptos means cargo-leptos will hot-reload this stylesheet
        <Stylesheet id="leptos" href="/pkg/leptos-test.css"/>

        // sets the document title
        <Title text="Welcome to Leptos"/>

        // content for this welcome page
        <Router>
            <main class="text-center">
                <Routes fallback=|| "Page not found.".into_view()>
                    <Route path=StaticSegment("") view=HomePage/>
                </Routes>
            </main>
        </Router>
    }
}

/// Server-side increment function
#[server]
pub async fn increment(count: i32) -> Result<i32, ServerFnError> {
    Ok(count + 1) // Increment logic handled server-side
}

/// Renders the home page of your application.
#[component]
fn HomePage() -> impl IntoView {
    // Client-side signal for count
    let count = RwSignal::new(0);

    // Button click handler to call the server function
    let on_click = move |_| {
        let count_value = count.get();
        spawn_local(async move { count.set(increment(count_value).await.unwrap()) });
    };

    view! {
        <h1 class="text-2xl font-bold mb-2">"Welcome to Leptos!"</h1>
        <button class="bg-gray-200 p-2 rounded-md hover:bg-gray-300" on:click=on_click>
            "Click Me: " <span class="font-bold">{move || count.get()}</span>
        </button>
    }
}
